# Generated by roxygen2: do not edit by hand

S3method(forecast,mdl_icomb_lst)
S3method(glance,mdl_icomb_lst)
export(forecast)
export(glance)
export(icomb)
export(lambda_path)
importFrom(dplyr,full_join)
importFrom(dplyr,mutate)
importFrom(fabletools,distribution_var)
importFrom(fabletools,response)
importFrom(future.apply,future_sapply)
importFrom(generics,forecast)
importFrom(generics,glance)
importFrom(glmnet,glmnet)
importFrom(purrr,exec)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,reduce)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,predict)
importFrom(tsibble,index_var)
importFrom(tsibble,interval)
importFrom(vctrs,vec_data)
